package com.atguigu.tingshu.album.service;

import com.atguigu.tingshu.model.album.AlbumInfo;
import com.atguigu.tingshu.query.album.AlbumInfoQuery;
import com.atguigu.tingshu.vo.album.AlbumInfoVo;
import com.atguigu.tingshu.vo.album.AlbumListVo;
import com.atguigu.tingshu.vo.album.AlbumStatVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

public interface AlbumInfoService extends IService<AlbumInfo> {


    /**
     * 保存专辑
     * @param albumInfoVo
     * @param userId
     */
    void saveAlbumInfo(AlbumInfoVo albumInfoVo, Long userId);


    /**
     * 保存专辑统计信息
     * @param albumId 专辑ID
     * @param statType 统计类型
     * @param statNum 统计数值 0401-播放量 0402-订阅量 0403-购买量 0403-评论数'
     */
    void saveAlbumInfoStat(Long albumId, String statType, int statNum);

    /**
     * 分页条件查询当前登录用户专辑列表
     * @param pageInfo
     * @param albumInfoQuery
     * @return
     */
    Page<AlbumListVo> findUserAlbumPage(Page<AlbumListVo> pageInfo, AlbumInfoQuery albumInfoQuery);

    /**
     * 根据专辑ID删除专辑相关信息
     * @param id
     * @return
     */
    void removeAlbumInfo(Long id);

    /**
     * 获取专辑信息，利用缓存提高效率同时采用分布式避免缓存击穿
     * @param id
     * @return
     */
    //AlbumInfo getAlbumInfo(Long id);
    /**
     * 根据专辑ID查询专辑信息，包含标签列表
     * @param id
     * @return
     */
    AlbumInfo getAlbumInfoFromDB(Long id);

    /**
     * 更新专辑信息
     * @param id
     * @param albumInfoVo
     * @return
     */
    void updateAlbumInfo(Long id, AlbumInfoVo albumInfoVo);

    /**
     * 查询当前登录用户发布专辑列表
     * @param userId
     * @return
     */
    List<AlbumInfo> findUserAllAlbumList(Long userId);

    /**
     * 根据专辑ID查询专辑统计信息
     * @param albumId
     * @return
     */
    AlbumStatVo getAlbumStatVo(Long albumId);


}
