package com.atguigu.tingshu.order.api;

import com.atguigu.tingshu.common.login.GuiGuLogin;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.order.service.OrderInfoService;
import com.atguigu.tingshu.vo.order.OrderInfoVo;
import com.atguigu.tingshu.vo.order.TradeVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@Tag(name = "订单管理")
@RestController
@RequestMapping("api/order")
@SuppressWarnings({"all"})
public class OrderInfoApiController {

	@Autowired
	private OrderInfoService orderInfoService;


	/**
	 * 对购买商品（VIP会员、专辑、声音）进行结算，渲染结算页
	 * @param tradeVo
	 * @return
	 */
	@GuiGuLogin
	@Operation(summary = "对购买商品（VIP会员、专辑、声音）进行结算，渲染结算页")
	@PostMapping("/orderInfo/trade")
	public Result<OrderInfoVo> trade(@RequestBody @Validated TradeVo tradeVo) {
		//1.获取用户ID
		Long userId = AuthContextHolder.getUserId();
		//2.调用业务逻辑处理不同商品类别结算页数据
		OrderInfoVo orderInfoVo = orderInfoService.trade(userId, tradeVo);
		return Result.ok(orderInfoVo);
	}


	/**
	 * 提交订单
	 * @param orderInfoVo
	 * @return
	 */
	@GuiGuLogin
	@Operation(summary = "提交订单")
	@PostMapping("/orderInfo/submitOrder")
	public Result<Map<String, String>> submitOrder(@RequestBody @Validated OrderInfoVo orderInfoVo) {
		//1.获取用户ID
		Long userId = AuthContextHolder.getUserId();
		//2.调用业务逻辑处理订单提交
		String orderNo = orderInfoService.submitOrder(userId, orderInfoVo);
		//3.返回对象封装订单编号
		Map<String, String> map = new HashMap<>();
		map.put("orderNo", orderNo);
		return Result.ok(map);
	}

}

