package com.atguigu.tingshu.search.service;

import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import com.atguigu.tingshu.model.search.AlbumInfoIndex;
import com.atguigu.tingshu.model.search.SuggestIndex;
import com.atguigu.tingshu.query.search.AlbumIndexQuery;
import com.atguigu.tingshu.vo.album.TrackStatMqVo;
import com.atguigu.tingshu.vo.search.AlbumInfoIndexVo;
import com.atguigu.tingshu.vo.search.AlbumSearchResponseVo;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public interface SearchService {


    /**
     * 将指定专辑导入索引库
     * @param albumId
     */
    void upperAlbum(Long albumId);

    /**
     * 下架索引库专辑
     * @param albumId
     */
    void lowerAlbum(Long albumId);


    /**
     * 多条件条件分页检索
     * @param albumIndexQuery
     * @return
     */
    AlbumSearchResponseVo search(AlbumIndexQuery albumIndexQuery);

    /**
     * 基于检索条件封装完整检索请求对象
     * @param albumIndexQuery
     * @return
     */
    SearchRequest buildDSL(AlbumIndexQuery albumIndexQuery);

    /**
     * 解析ES响应结果封装自定义VO
     * @param searchResponse
     * @param albumIndexQuery
     * @return
     */
    AlbumSearchResponseVo parseResult(SearchResponse<AlbumInfoIndex> searchResponse, AlbumIndexQuery albumIndexQuery);

    /**
     * 得到置顶三级分类热门专辑TOP6列表
     * @param category1Id
     * @return
     */
    List<Map<String, Object>> getBaseCategoryListByCategory1Id(Long category1Id);

    /**
     * 将专辑名称存入提词索引库
     * @param albumInfoIndex
     */
    void saveSuggest(AlbumInfoIndex albumInfoIndex);

    /**
     * 根据用户已录入字符返回提示词列表,自动补全效果
     * @param keyword
     * @return
     */
    List<String> completeSuggest(String keyword);

    /**
     * 解析建议词结果
     * @param suggestIndexSearchResponse ES响应对象
     * @param suggestName 自定义名称
     * @return
     */
    Collection<String> parseSuggestResult(SearchResponse<SuggestIndex> suggestIndexSearchResponse, String suggestName);

    /**
     * 手动更新Redis中小时榜数据
     */
    void updateLatelyAlbumRanking();

    List<AlbumInfoIndexVo> getRankingList(Long category1Id, String dimension);

    void updateAlbumStat(TrackStatMqVo trackStatMqVo);
}
