package com.atguigu.tingshu.user.api;

import com.atguigu.tingshu.common.login.GuiGuLogin;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.user.service.UserListenProcessService;
import com.atguigu.tingshu.vo.user.UserListenProcessVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Map;

@Tag(name = "用户声音播放进度管理接口")
@RestController
@RequestMapping("api/user")
@SuppressWarnings({"all"})
public class UserListenProcessApiController {

	@Autowired
	private UserListenProcessService userListenProcessService;


	/**
	 * 业务需求：用户未登录，收听某个声音，引导用户登录
	 * @param trackId
	 * @return
	 */
	@GuiGuLogin//(required = false)
	@Operation(summary = "获取声音播放进度")
	@GetMapping("/userListenProcess/getTrackBreakSecond/{trackId}")
	public Result<BigDecimal> getTrackBreakSecond(@PathVariable("trackId") Long trackId) {
		//获取当前用户ID
		Long userId = AuthContextHolder.getUserId();
		BigDecimal breakSecond = userListenProcessService.getTrackBreakSecond(userId, trackId);
		return Result.ok(breakSecond);
	}


	/**
	 * 业务需求：更新播放进度
	 * @param userListenProcessVo
	 * @return
	 */
	@GuiGuLogin
	@Operation(summary = "更新播放进度")
	@PostMapping("/userListenProcess/updateListenProcess")
	public Result updateListenProcess(@RequestBody UserListenProcessVo userListenProcessVo) {
		userListenProcessService.updateListenProcess(userListenProcessVo);
		return Result.ok();
	}



	/**
	 * 获取最近播放专辑及声音
	 * @return
	 */
	@GuiGuLogin
	@Operation(summary = "获取最近播放专辑及声音")
	@GetMapping("/userListenProcess/getLatelyTrack")
	public Result<Map<String, Long>> getLatelyTrack(){
		Long userId = AuthContextHolder.getUserId();
		Map<String, Long> map = userListenProcessService.getLatelyTrack(userId);
		return Result.ok(map);
	}
}

